@@ -98,7 +98,7 @@ rdpInputKeyboardEvent(rdpPtr dev, int msg,
                       long param1, long param2,
                       long param3, long param4)
 {
-    dev->last_event_time = time(0);
+    dev->last_event_time_ms = GetTimeInMillis();
 
     if (g_input_proc[0].proc != 0)
     {
@@ -113,7 +113,7 @@ rdpInputMouseEvent(rdpPtr dev, int msg,
                    long param1, long param2,
                    long param3, long param4)
 {
-    dev->last_event_time = time(0);
+    dev->last_event_time_ms = GetTimeInMillis();
 
     if (g_input_proc[1].proc != 0)
     {
